@@ -20,17 +20,24 @@ import email
 import traceback
 import dateparser
 from dateutil import tz
+import base64
+from bs4 import BeautifulSoup
 
 import smtplib
+from email.message import EmailMessage
 from email.mime.text import MIMEText
 from email.mime.multipart import MIMEMultipart
-from googleapiclient.discovery import build
 from google_auth_oauthlib.flow import InstalledAppFlow
 from google.auth.transport.requests import Request
+from googleapiclient.discovery import build
+from googleapiclient.errors import HttpError
 
 from .sqlhelper import _get_config, _get_credentials
 from .misc import file_age, verbose_display, _pycof_folders
 
+# Define default Google API scopes
+default_scopes = ['https://mail.google.com/', 'https://www.googleapis.com/auth/calendar.readonly']
+
 
 #######################################################################################################################
 
@@ -45,7 +52,6 @@ def send_email(to, subject, body, cc='', credentials={}):
         * **body** (:obj:`str`): Content of the email to be send.
         * **cc** (:obj:`str`): Email address to be copied (defaults None).
         * **credentials** (:obj:`dict`): Credentials to use to connect to the database. You can also provide the credentials path or the json file name from :obj:`/etc/.pycof/` (defaults {}).
-        * **verbose** (:obj:`bool`): Displays if the email was sent successfully (defaults False).
 
     :Configuration: The function requires the below arguments in the configuration file.
 
@@ -96,6 +102,194 @@ def send_email(to, subject, body, cc='', credentials={}):
     server.sendmail(config.get('EMAIL_USER'), [to, '', cc], text)
     server.quit()
 
+# Send an email from Gmail
+class google_email:
+    def __init__(self, credentials={}, scopes=default_scopes, token_path=None):
+        """Simplified class to send email from a Gmail email address with secured connection with developper API.
+        The `Google credentials file <https://developers.google.com/calendar/quickstart/python>`_ needs to be saved as :obj:`/etc/.pycof/google.json`.
+
+        :param credentials: Credentials to use which includes the from email address to display. See Setup, defaults to {}.
+        :type credentials: :obj:`dict`, optional
+        :param scopes: Targeted permissions required. Check https://developers.google.com/calendar/auth for more details, defaults to ['https://mail.google.com/', 'https://www.googleapis.com/auth/calendar.readonly'].
+        :type scopes: :obj:`list`, optional
+        :param token_path: Path to the saved `token.pickle` authentication file, defaults to None and saves in the PYCOF temporary data folder.
+        :type token_path: :obj:`str`, optional
+
+        :Configuration: The function requires a configuration file stored at :obj:`/etc/.pycof/google.json`.
+            This file can be generated at https://developers.google.com/calendar/quickstart/python.
+            User will need to enable the Google Calendar API on the account from Step 1.
+
+        :Example:
+            >>> _body = '<html><body><h1>This email is a test</h1><p>Hello world!</p></body></html>'
+            >>> pc.google_email().send(to='email@example.com', subject='Test', body=_body, cc='friend@example.com')
+            >>> # Alternative where we specify token_path
+            >>> pc.google_email(token_path='/home/ubuntu/token.pickle').send(to='email@example.com', subject='Test', body=_body, cc='friend@example.com')
+        """
+        self.scopes = scopes
+        self.token_path = os.path.join(_pycof_folders('data'), 'token.pickle') if token_path is None else token_path
+        self._creds = credentials
+
+    def _get_creds(self):
+        """Retreive Google credentials.
+
+        :return: Google calendar credentials.
+        :rtype: :obj:`google_auth_oauthlib`
+        """
+        creds = None
+        # The file token.pickle stores the user's access and refresh tokens, and is
+        # created automatically when the authorization flow completes for the first
+        # time.
+        creds_path = os.path.join(_pycof_folders('creds'), 'google.json')
+
+        if os.path.exists(self.token_path):
+            with open(self.token_path, 'rb') as token:
+                creds = pickle.load(token)
+        # If there are no (valid) credentials available, let the user log in.
+        if not creds or not creds.valid:
+            if creds and creds.expired and creds.refresh_token:
+                creds.refresh(Request())
+            else:
+                flow = InstalledAppFlow.from_client_secrets_file(creds_path, self.scopes)
+                creds = flow.run_local_server(port=0)
+            # Save the credentials for the next run
+            with open(self.token_path, 'wb') as token:
+                pickle.dump(creds, token)
+        return creds
+
+    def send(self, to, subject, body, cc='', return_status=False):
+        """Send the email.
+
+        :param to: Recipient of the email.
+        :type to: :obj:`str`
+        :param subject: Subject of the email.
+        :type subject: :obj:`str`
+        :param body: Content of the email to be send.
+        :type body: :obj:`str`
+        :param cc: Email address to be copied, defaults to ''.
+        :type cc: :obj:`str`, optional
+        :param return_status: Returns status of the email sent, defaults to False.
+        :type return_status: :obj:`bool`, optional
+
+        :return: If `return_status=True`, returns a dictionnary with the status of the email.
+        :rtype: :obj:`dict`
+        """
+        config = _get_config(self._creds)
+        try:
+            service = build('gmail', 'v1', credentials=self._get_creds())
+            msg = MIMEMultipart()
+            msg['from'] = config.get('EMAIL_SENDER')
+            msg['to'] = to
+            msg['cc'] = '' if cc == '' else cc
+            msg['subject'] = subject
+
+            mail_type = 'html' if '</' in body else 'plain'
+            msg.attach(MIMEText(body, mail_type))
+
+            create_message = {
+                "raw": base64.urlsafe_b64encode(bytes(msg.as_string(), "utf-8")).decode("utf-8")
+            }
+            # pylint: disable=E1101
+            send_message = (service.users().messages().send(userId="me", body=create_message).execute())
+        except HttpError as error:
+            print(F'An error occurred: {error}')
+            send_message = None
+        if return_status:
+            return send_message
+
+    def getEmails(self, nb_email=1, _to='me'):
+        """Get latest emails from your Gmail address.
+
+        :param nb_email: Number of emails to retreive, defaults to 1.
+        :type nb_email: :obj:`int`, optional
+        :param _to: User ID to use to retreive emails, defaults to 'me'.
+        :type _to: :obj:`str`, optional
+
+        :example:
+            >>> pycof.GetEmails(2)
+            ... +----------------------------+-----------------+----------------+---------------+
+            ... |                       Date |            From |        Subject |            To |
+            ... +----------------------------+-----------------+----------------+---------------+
+            ... |  2021-01-01 04:00:03+01:00 | test@domain.com |          Testo |  me@gmail.com |
+            ... |  2021-01-01 03:14:09+01:00 | test@domain.com |   Another test |  me@gmail.com |
+            ... +----------------------------+-----------------+----------------+---------------+
+
+        :return: Data frame with last emails.
+        :rtype: :obj:`pandas.DataFrame`
+        """
+        creds = self._get_creds()
+
+        # Connect to the Gmail API
+        service = build('gmail', 'v1', credentials=creds)
+
+        # request a list of all the messages
+        result = service.users().messages().list(userId=_to).execute()
+
+        # We can also pass maxResults to get any number of emails. Like this:
+        # result = service.users().messages().list(maxResults=200, userId='me').execute()
+        messages = result.get('messages')
+
+        # messages is a list of dictionaries where each dictionary contains a message id.
+
+        df = []
+
+        # iterate through all the messages
+        for msg in messages[:nb_email]:
+            # Get the message from its id
+            txt = service.users().messages().get(userId=_to, id=msg['id']).execute()
+
+            # Use try-except to avoid any Errors
+            try:
+                # Get value of 'payload' from dictionary 'txt'
+                payload = txt['payload']
+                headers = payload['headers']
+
+                # Look for Subject and Sender Email in the headers
+                for d in headers:
+                    if d['name'] == 'subject':
+                        _subj = d['value']
+                    if d['name'] == 'From':
+                        _from = d['value']
+                    if d['name'] == 'to':
+                        _dest = d['value']
+                    if d['name'] == 'Date':
+                        ddt = dateparser.parse(d['value'].strip())
+                        try:
+                            _date = ddt.replace(tzinfo=datetime.timezone.utc).astimezone(tz=tz.tzlocal())
+                        except Exception:
+                            _date = np.nan
+
+                # Get email attachments
+                i = 1
+                attch = {}
+                _body = ''
+                for part in payload.get('parts'):
+                    if part['filename']:
+                        att_id = part['body']['attachmentId']
+                        att = service.users().messages().attachments().get(userId=_to, messageId=msg['id'], id=att_id).execute()
+                        data = att['data']
+                        file_data = base64.urlsafe_b64decode(data.encode('UTF-8'))
+                        filePath = os.path.join(_pycof_folders('data'), part['filename'])
+                        attch.update({f'Attachment {i}': filePath})
+                        # Download the attachment
+                        with open(filePath, 'wb') as f:
+                            f.write(file_data)
+                        i += 1
+                    else:
+                        data = part['body']['data']
+                        data = data.replace("-", "+").replace("_", "/")
+                        decoded_data = base64.b64decode(data)
+
+                        # Now, the data obtained is in lxml. So, we will parse
+                        # it with BeautifulSoup library
+                        soup = BeautifulSoup(decoded_data, "lxml")
+                        _body = soup.body()
+                for_df = {'From': _from, 'To': _dest, 'Date': _date, 'Subject': _subj}  # , 'Body': _body
+                for_df.update(attch)
+                df += [pd.DataFrame(for_df, index=[0])]
+            except Exception:
+                pass
+        return pd.concat(df).reset_index(drop=True)
+
 
 #######################################################################################################################
 
@@ -291,7 +485,7 @@ def str2bool(value):
 
 # Getting Google Calendar events
 class GoogleCalendar:
-    def __init__(self, timezone='Europe/Paris', scopes=['https://www.googleapis.com/auth/calendar.readonly'], temp_folder=None):
+    def __init__(self, timezone='Europe/Paris', scopes=default_scopes, token_path=None):
         """Get all available events on a Google Calendar.
         The `Google credentials file <https://developers.google.com/calendar/quickstart/python>`_ needs to be saved as :obj:`/etc/.pycof/google.json`.
 
@@ -299,8 +493,8 @@ class GoogleCalendar:
         :type timezone: :obj:`str`, optional
         :param scopes: Targeted permissions required. Check https://developers.google.com/calendar/auth for more details, defaults to ['https://www.googleapis.com/auth/calendar.readonly'].
         :type scopes: :obj:`list`, optional
-        :param temp_folder: Folder in which we will save the `token.pickle` authentication file, defaults to None and saves in the PYCOF temporary data folder.
-        :type temp_folder: :obj:`str`, optional
+        :param token_path: Path to the saved `token.pickle` authentication file, defaults to None and saves in the PYCOF temporary data folder.
+        :type token_path: :obj:`str`, optional
 
         :Configuration: The function requires a configuration file stored at :obj:`/etc/.pycof/google.json`.
             This file can be generated at https://developers.google.com/calendar/quickstart/python.
@@ -308,7 +502,7 @@ class GoogleCalendar:
         """
         self.timezone = pytz.timezone(timezone)
         self.scopes = scopes
-        self.data_fold = _pycof_folders('data') if temp_folder is None else temp_folder
+        self.token_path = os.path.join(_pycof_folders('data'), 'token.pickle') if token_path is None else token_path
 
     def _get_creds(self):
         """Retreive Google credentials.
@@ -320,11 +514,10 @@ class GoogleCalendar:
         # The file token.pickle stores the user's access and refresh tokens, and is
         # created automatically when the authorization flow completes for the first
         # time.
-        token_path = os.path.join(self.data_fold, 'token.pickle')
         creds_path = os.path.join(_pycof_folders('creds'), 'google.json')
 
-        if os.path.exists(token_path):
-            with open(token_path, 'rb') as token:
+        if os.path.exists(self.token_path):
+            with open(self.token_path, 'rb') as token:
                 creds = pickle.load(token)
         # If there are no (valid) credentials available, let the user log in.
         if not creds or not creds.valid:
@@ -334,7 +527,7 @@ class GoogleCalendar:
                 flow = InstalledAppFlow.from_client_secrets_file(creds_path, self.scopes)
                 creds = flow.run_local_server(port=0)
             # Save the credentials for the next run
-            with open(token_path, 'wb') as token:
+            with open(self.token_path, 'wb') as token:
                 pickle.dump(creds, token)
         return creds
 
